import logging
import copy
from awsibox import cfg
from ..RP import RP_to_cfg, merge_dict
from troposphere import (
    Parameter,
    Output,
    Ref,
)
from ..shared import (
    auto_get_props,
    add_obj,
    do_no_override,
    get_condition,
    parse_ibox_key,
    ibox_eval,
)


def Joker(key, module, cls):
    for n, v in getattr(cfg, key).items():
        if key == "Condition":
            # Condition are just dict, directly add them
            do_no_override(True)
            add_obj({n: ibox_eval(v)})
            do_no_override(False)
            continue

        if not v.get("IBOX_ENABLED", True):
            continue
        if not ibox_eval(v.get("IBOX_ENABLED_IF", "True")):
            continue

        parse_ibox_key_conf = {"IBOX_INDEXNAME": n}
        resname = f"{key}{n}"
        mapname = resname

        # change resource name using key IBOX_RESNAME
        ibox_resname = v.get("IBOX_RESNAME")
        if ibox_resname:
            resname = parse_ibox_key(ibox_resname, parse_ibox_key_conf)

        if cfg.debug:
            logging.error(f"Joker processing: {resname}")

        # get IBOX_LINKED_OBJ keys
        linked_obj_name = v.get("IBOX_LINKED_OBJ_NAME", "")
        linked_obj_index = v.get("IBOX_LINKED_OBJ_INDEX", "")

        if module:
            mod = __import__(f"troposphere.{module}")
            my_module = getattr(mod, module)
        else:
            # for classes in troposphere __init__
            my_module = __import__("troposphere")
        my_class = getattr(my_module, cls)

        obj = my_class(resname)

        # use IBOX_SOURCE_OBJ to prepopulate obj
        ibox_source_obj = v.get("IBOX_SOURCE_OBJ", [])
        if isinstance(ibox_source_obj, str):
            ibox_source_obj = [ibox_source_obj]
        for source_obj in ibox_source_obj:
            source_obj = parse_ibox_key(source_obj, parse_ibox_key_conf)
            source_obj_conf = copy.deepcopy(getattr(cfg, source_obj))
            RP_to_cfg(
                source_obj_conf,
                prefix=mapname,
                overwrite=False,
                mappedvalues=cfg.mappedvalues,
                check_mapped=True,
            )
            v = merge_dict(v, source_obj_conf, keep=True)

            # Uncomment to use old method: auto_get_props
            # auto_get_props(obj, mapname=source_obj, indexname=n)
            # #reset obj title, if changed by IBOX_TITLE key
            # obj.title = resname

        # autocreate condition
        if v.get("Create"):
            add_obj(
                Parameter(
                    f"{resname}Create",
                    Description=f"Create {resname}",
                    AllowedValues=["", "yes", "no"],
                    Type="String",
                    Default="",
                )
            )
            if "Create.IBOX_AUTO_PO" not in v:
                add_obj(get_condition(resname, "equals", "yes", f"{resname}Create"))
                add_obj(
                    Output(
                        resname,
                        Condition=resname,
                        Value=Ref(v.get("IBOX_TITLE", resname)),
                    )
                )
                if not hasattr(obj, "Condition"):
                    obj.Condition = resname

        # populate obj
        auto_get_props(
            obj,
            mapname=mapname,
            indexname=n,
            linked_obj_name=linked_obj_name,
            linked_obj_index=linked_obj_index,
        )

        add_obj(obj)



INSTANCE_SIZES = [
    "nano",
    "micro",
    "small",
    "medium",
    "large",
    "xlarge",
    "2xlarge",
    "4xlarge",
    "8xlarge",
    "12xlarge",
    "16xlarge",
    "24xlarge",
    "32xlarge",
    "48xlarge",
]

INSTANCE_FAMILY = [
    {
        "Name": "t2",
        "Min": "nano",
        "Max": "2xlarge",
        "RDS": True,
    },
    {
        "Name": "m4",
        "Min": "large",
        "Max": "4xlarge",
        "RDS": True,
    },
    {
        "Name": "c4",
        "Min": "large",
        "Max": "4xlarge",
    },
    {
        "Name": "r4",
        "Min": "large",
        "Max": "4xlarge",
        "RDS": True,
    },
    {
        "Name": "t3",
        "Min": "nano",
        "Max": "2xlarge",
        "RDS": True,
    },
    {
        "Name": "m5",
        "Min": "large",
        "Max": "4xlarge",
        "RDS": True,
    },
    {
        "Name": "c5",
        "Min": "large",
        "Max": "4xlarge",
    },
    {
        "Name": "r5",
        "Min": "large",
        "Max": "4xlarge",
        "RDS": True,
    },
    {
        "Name": "t3a",
        "Min": "nano",
        "Max": "2xlarge",
    },
    {
        "Name": "t4g",
        "Min": "nano",
        "Max": "2xlarge",
        "RDS": True,
    },
    {
        "Name": "m5a",
        "Min": "large",
        "Max": "4xlarge",
    },
    {
        "Name": "c5a",
        "Min": "large",
        "Max": "4xlarge",
    },
    {
        "Name": "r5a",
        "Min": "large",
        "Max": "4xlarge",
    },
    {
        "Name": "m6i",
        "Min": "large",
        "Max": "4xlarge",
        "RDS": True,
    },
    {
        "Name": "c6i",
        "Min": "large",
        "Max": "4xlarge",
    },
    {
        "Name": "r6i",
        "Min": "large",
        "Max": "4xlarge",
        "RDS": True,
    },
    {
        "Name": "m6a",
        "Min": "large",
        "Max": "4xlarge",
    },
    {
        "Name": "c6a",
        "Min": "large",
        "Max": "4xlarge",
    },
    {
        "Name": "c7g",
        "Min": "large",
        "Max": "4xlarge",
    },
    {
        "Name": "m7g",
        "Min": "large",
        "Max": "4xlarge",
        "RDS": True,
    },
]

# currently nearly empty as no more used with newer generations instances
EPHEMERAL_MAP = {
    "InstaceEphemeral0": ["m3.medium", "m3.xlarge"],
    "InstaceEphemeral1": ["m3.xlarge"],
    "InstaceEphemeral2": [""],
}

# build instances list
def build_instance_list():
    family_instances_list = []
    family_instances_list_rds = []
    for family in INSTANCE_FAMILY:
        name = family["Name"]
        min_size = INSTANCE_SIZES.index(family["Min"])
        max_size = INSTANCE_SIZES.index(family["Max"])

        for s in INSTANCE_SIZES[min_size : max_size + 1]:
            family_instances_list.append(f"{name}.{s}")
            if family.get("RDS"):
                family_instances_list_rds.append(f"db.{name}.{s}")

    return ["default"] + family_instances_list, [""] + family_instances_list_rds


INSTANCE_LIST, INSTANCE_LIST_RDS = build_instance_list()



from troposphere import cloudfront
from awsibox import cfg
from ..shared import (
    auto_get_props,
    add_obj,
    clf_compute_order,
)


def process_cache_policy(v):
    # Use CachePolicyId/OriginRequestPolicyId or legacy mode
    if "CachePolicyId" in v:
        for k in ["DefaultTTL", "MaxTTL", "MinTTL", "ForwardedValues"]:
            try:
                del v[k]
            except Exception:
                pass


def CF_CloudFront(key):
    for n, v in getattr(cfg, key).items():
        if not v.get("IBOX_ENABLED", True):
            continue
        resname = f"{key}{n}"
        # Resources
        R_CloudFrontDistribution = cloudfront.Distribution(resname)
        distribution_config = v["DistributionConfig"]

        # process cache behaviors
        process_cache_policy(distribution_config["DefaultCacheBehavior"])
        for m, w in distribution_config["CacheBehaviors"].items():
            process_cache_policy(w)

        # process origins
        for m, w in distribution_config["Origins"].items():
            if "VpcOriginId" not in w["VpcOriginConfig"]:
                del w["VpcOriginConfig"]
            if any(n in w for n in ["S3OriginConfig", "VpcOriginConfig"]):
                del w["CustomOriginConfig"]

        # Automatically compute Behaviour Order based on PathPattern
        cfg.dbg_clf_compute_order = {}
        sortedcachebehaviors = sorted(
            distribution_config["CacheBehaviors"].items(),
            key=lambda x_y: clf_compute_order(x_y[1]["PathPattern"]),
        )
        distribution_config["CacheBehaviors"] = {
            x[0]: x[1] for x in sortedcachebehaviors
        }

        if cfg.debug:
            print("##########CLF_COMPUTE_ORDER#########START#######")
            for n, v in iter(
                sorted(cfg.dbg_clf_compute_order.items(), key=lambda x_y: x_y[1])
            ):
                print(f"{n} {v}\n")
            print("##########CLF_COMPUTE_ORDER#########END#######")

        auto_get_props(R_CloudFrontDistribution, mapname=resname)
        add_obj(R_CloudFrontDistribution)



import copy
from troposphere import cloudformation, Ref
from awsibox import cfg
from ..shared import get_endvalue, add_obj, auto_get_props


def mapping_EnvRegion():
    RP = copy.deepcopy(cfg.RP_map)

    for env in list(RP):
        rvalue = RP[env]
        for region in list(rvalue.keys()):
            for key in cfg.mappedvalues:
                if key not in RP[env][region]:
                    # get base/global value
                    try:
                        RP[env][region][key] = getattr(cfg, key)
                    except Exception:
                        pass
            # delete empty mappings CF do not like them
            if not RP[env][region]:
                del RP[env][region]
        if not RP[env]:
            del RP[env]

    return RP


def mapping_EC2():
    mappings = {}
    mappings["InstanceTypes"] = {}

    for i in cfg.INSTANCE_LIST:
        mappings["InstanceTypes"].update(
            {
                i: {
                    "InstaceEphemeral0": "1"
                    if i in cfg.EPHEMERAL_MAP["InstaceEphemeral0"]
                    else "0",
                    "InstaceEphemeral1": "1"
                    if i in cfg.EPHEMERAL_MAP["InstaceEphemeral1"]
                    else "0",
                    "InstaceEphemeral2": "1"
                    if i in cfg.EPHEMERAL_MAP["InstaceEphemeral2"]
                    else "0",
                }
            }
        )

    return mappings


def mapping_AZ():
    mappings = {}
    mappings["AvabilityZones"] = {}
    AZ = cfg.AZones

    for r in cfg.regions:
        zones = {}
        try:
            nzones = AZ[r]
        except Exception:
            nzones = AZ["default"]

        try:
            nzones = cfg.RP["dev"][r]["AZones"]
        except Exception:
            pass

        for n in range(AZ["MAX"]):
            zones[f"Zone{n}"] = "True" if nzones > n else "False"

        mappings["AvabilityZones"][r] = zones

    return mappings


def CFM_Mappings(key):
    for n, v in getattr(cfg, key).items():
        if n == "IBoxEnvRegion":
            c_Mapping = mapping_EnvRegion()
        elif n == "IBoxEC2":
            c_Mapping = mapping_EC2()
        elif n == "IBoxAZ":
            c_Mapping = mapping_AZ()
        else:
            c_Mapping = {n: v}

        cfg.Mappings.update(c_Mapping)


def CFM_CustomResourceReplicator(key):
    R_Replicator = cloudformation.CustomResource(
        "CloudFormationCustomResourceStackReplicator"
    )
    auto_get_props(R_Replicator)

    for p, v in cfg.Parameters.items():
        if not p.startswith("Env"):
            value = get_endvalue(p)
        else:
            value = Ref(p)
        setattr(R_Replicator, p, value)

    add_obj(R_Replicator)



import os
import sys
from functools import reduce
from .cfg_instance_types import INSTANCE_LIST, INSTANCE_LIST_RDS, EPHEMERAL_MAP

cwd = os.getcwd()
sys.path.append(os.path.join(cwd, "lib"))

no_override = False

Parameters = {}
Conditions = {}
Mappings = {}
Resources = {}
Outputs = {}
Metadata = {}

OBJS = {}

YAML_COMMON_NO_BRAND = []

APP_DIR = os.path.dirname(os.path.realpath(__file__))

PATH_INT = os.path.join(APP_DIR, "cfg")
PATH_INT = os.path.normpath(PATH_INT)

PATH_EXT = os.path.join(cwd, "cfg")
PATH_EXT = os.path.normpath(PATH_EXT)

IBOX_BASE_KEY_NAME = "IBOX_BASE"

IBOX_BRAND_DIR = "ibox"
STACKS_DIR = "stacks"

STACK_TYPES = [
    "agw",
    "cch",
    "clf",
    "ec2",
    "ecs",
    "rds",
    "res",
    "tsk",
    "lbd",
]

MAX_SECURITY_GROUPS = 4
# SECURITY_GROUPS_DEFAULT equals list of empty values
# (Ex. for "MAX_SECURITY_GROUPS = 3" we have "SECURITY_GROUPS_DEFAULT = ',,'")
SECURITY_GROUPS_DEFAULT = reduce(
    lambda a, i: f",{a}", list(range(MAX_SECURITY_GROUPS - 1)), ""
)

ENV_BASE = ["dev", "stg", "prd"]

DEFAULT_REGIONS = [
    "eu-west-1",
    "us-east-1",
]

AZones = {
    "MAX": 6,
    "default": 3,
    "us-east-1": 6,
    "us-west-1": 2,
    "us-west-2": 4,
    "ca-central-1": 2,
    "sa-east-1": 2,
    "cn-north-1": 2,
    "ap-northeast-3": 1,
}

AZoneNames = ["A", "B", "C", "D", "E", "F"]

VPC_DEFAULT_CIDR_BLOCK = "10.80.0.0/16"
VPC_DEFAULT_CIDR_BLOCK_PREFIX = ".".join(VPC_DEFAULT_CIDR_BLOCK.split(".")[0:2])
VPC_DEFAULT_SUBNETS_CIDR_BLOCK_PRIVATE = [
    f"{VPC_DEFAULT_CIDR_BLOCK_PREFIX}.{i * 16}.0/20" for i in range(AZones["MAX"])
]
VPC_DEFAULT_SUBNETS_CIDR_BLOCK_PUBLIC = [
    f"{VPC_DEFAULT_CIDR_BLOCK_PREFIX}.{i + 200}.0/24" for i in range(AZones["MAX"])
]

TROPO_CLASS_TO_CFM = {
    "AWS::WAFv2::WebACL": {
        "WebACLRule": "Rule",
    },
    "AWS::ECR::ReplicationConfiguration": {
        "ReplicationConfigurationProperty": "ReplicationConfiguration"
    },
}

EVAL_TROPO_FUNCTIONS_IN_CFG = (
    "Sub(",
    "Ref(",
    "GetAtt(",
    "Split(",
    "Select(",
    "Export(",
    "ImportValue(",
    "Join(",
    "Base64(",
    "If(",
    "Equals(",
    "Not(",
    "GetAZs(",
    "Tags(",
    "FindInMap(",
)

EVAL_PYTHON_FUNCTIONS_IN_CFG = (
    "cfg.",
    "get_expvalue(",
    "get_subvalue(",
    "get_endvalue(",
    "get_resvalue(",
    "get_condition(",
    "dict(",
    "eval(",
    "str(",
    "int(",
    "list(",
    "getattr(",
    "range(",
)

EVAL_FUNCTIONS_IN_CFG = EVAL_PYTHON_FUNCTIONS_IN_CFG + EVAL_TROPO_FUNCTIONS_IN_CFG

CLF_PATH_PATTERN_REPLACEMENT = {
    "/": "SLASH",
    "*": "STAR",
    "-": "HYPH",
    "?": "QUEST",
    ".": "DOT",
    "_": "USCORE",
}

MERGE_RP_KEEP_AS_LIST = ("IBOX_LINKED_OBJ",)

# override previous cfg with an External one
try:
    with open(os.path.join(cwd, "lib", "cfgExt.py")) as f:
        exec(f.read())
except FileNotFoundError:
    pass


class BuildEnvs(dict):
    def __init__(self, *arg, **kw):
        super().__init__(*arg, **kw)

    def clear(self):
        self.__dict__.clear()
        return super().clear()

    def __setattr__(self, key, item):
        super().__setattr__(key, item)
        super().__setitem__(key, item)


CFG_TO_FUNC_RENAME = {
    "lambda": "awslambda",
    "AutoScalingAutoScalingGroup": "AutoScalingGroup",
    "LambdaFunction": "Lambda",
    "CloudFrontOriginAccessControl": "CloudFrontOriginOriginAccessControl",
    "Subscription": "SubscriptionResource",
    "Route53ResolverResolverEndpoint": "Route53ResolverEndpoint",
    "Route53ResolverResolverRule": "Route53ResolverRule",
    "Route53ResolverResolverRuleAssociation": "Route53ResolverRuleAssociation",
}

# Order is VERY important do not CHANGE it!
CFG_TO_FUNC_OVERRIDE = {
    "Parameter": {
        "module": "joker",
        "func": (None, "Parameter"),
    },
    "Condition": {
        "module": "joker",
        "func": (None, "Condition"),
    },
    "Mapping": {
        "module": "cloudformation",
        "func": "CFM_Mappings",
    },
    "ApiGatewayDeployment": {
        "dep": ["ApiGatewayStage"],
    },
    "AutoScalingGroup": {
        "module": "autoscaling",
        "func": "AS_Autoscaling",
        "dep": ["SecurityGroups", "ECSCapacityProvider"],
    },
    "CloudFormationCustomResource": {
        "dep": ["ECSService", "ElasticLoadBalancingV2Listener"],
    },
    "CloudFrontDistribution": {
        "module": "cloudfront",
        "func": "CF_CloudFront",
        "dep": [
            "ElasticLoadBalancingV2Listener",
            "ElasticLoadBalancingLoadBalancer",
            "CloudFrontLambdaFunctionAssociation",
        ],
    },
    "CloudFrontOriginAccessIdentity": {
        "dep": ["S3Bucket"],
    },
    # This is not a resource but a sub-resource
    "CloudFrontLambdaFunctionAssociation": {
        "module": "joker",
        "func": ("cloudfront", "LambdaFunctionAssociation"),
    },
    "CloudFrontVpcOrigin": {
        "func": ("cloudfront", "VpcOrigin"),
        "module": "joker",
        "dep": ["ElasticLoadBalancingV2Listener"],
    },
    "CloudWatchAlarm": {
        "dep": ["ElasticLoadBalancingLoadBalancer", "SQSQueue", "Lambda"],
    },
    "EC2SecurityGroup": {
        "module": "ec2",
        "func": "SG_SecurityGroup",
        "dep": ["ECSTaskDefinition", "EFSFileSystem"],
    },
    "EC2SecurityGroupIngress": {
        "module": "ec2",
        "func": "SG_SecurityGroupIngresses",
        "dep": [
            "ElasticLoadBalancingV2Listener",
            "ElasticLoadBalancingLoadBalancer",
            "EFSFileSystem",
        ],
    },
    "EC2Subnet": {
        "dep": ["EC2VPC"],
    },
    "EC2SubnetRouteTableAssociation": {
        "dep": ["EC2VPC"],
    },
    "ECSService": {
        "dep": ["SecurityGroups", "ECSTaskDefinition"],
    },
    "ElasticLoadBalancingV2Listener": {
        "dep": ["ElasticLoadBalancingLoadBalancer"],
    },
    "ElasticLoadBalancingV2TargetGroup": {
        # need this for override on ContainerDefinitions1ContainerPort
        "dep": ["ECSTaskDefinition"],
    },
    "EventsRule": {
        "dep": ["SecurityGroups"],
    },
    "IAMPolicy": {
        "func": ("iam", "PolicyType"),
        "module": "joker",
        "dep": ["S3Bucket"],
    },
    "IAMRole": {
        "dep": ["Lambda", "S3Bucket"],
    },
    "LambdaLayerVersionPermission": {
        "dep": ["LambdaLayerVersion"],
    },
    "LambdaPermission": {
        "dep": ["SNSSubscription", "Lambda", "EventsRule", "SchedulerSchedule"],
    },
    "LambdaVersion": {
        "dep": ["Lambda"],
    },
    "LogsLogGroup": {
        "dep": ["Lambda"],
    },
    "RDSDBParameterGroup": {
        "dep": ["RDSDBInstance"],
    },
    "Route53RecordSet": {
        "func": ("route53", "RecordSetType"),
        "module": "joker",
        "dep": [
            "ApiGatewayDomainName",
            "DBInstance",
            "EFSFileSystem",
            "ElasticLoadBalancingLoadBalancer",
            "CloudFrontDistribution",
        ],
    },
    "S3BucketPolicy": {
        "dep": ["S3Bucket"],
    },
    "SecurityGroups": {
        "module": "ec2",
        "func": "SG_SecurityGroups",
        "dep": ["ElasticLoadBalancingLoadBalancer"],
    },
    "SchedulerSchedule": {
        "dep": ["SecurityGroups"],
    },
    "SQSQueuePolicy": {
        "dep": ["SNSSubscription"],
    },
    # ReplicateRegions need to be the last one
    "CCRReplicateRegions": {
        "module": "cloudformation",
        "func": "CFM_CustomResourceReplicator",
    },
    # Output need to be last line
    "Output": {
        "module": "joker",
        "func": (None, "Output"),
    },
}



from troposphere import (
    sso,
    rds,
    ec2,
    elasticloadbalancing,
    elasticloadbalancingv2,
    validators,
    AWSHelperFn,
    If,
    Parameter,
    Tags,
)


# Temporary fix for https://github.com/cloudtools/troposphere/issues/2146
sso.PermissionSet.props["InlinePolicy"] = (str, False)


# set Parameter default values
def my_parameter_init(self, title, **kwargs):
    super(Parameter, self).__init__(title, **kwargs)
    if "Type" not in kwargs:
        self.Type = "String"
    if "Default" not in kwargs:
        self.Default = ""


Parameter.__init__ = my_parameter_init


# Temporary fix for https://github.com/cloudtools/troposphere/issues/1474
def my_one_of(class_name, properties, property, conditionals):
    if properties.get(property) not in conditionals and not isinstance(
        properties.get(property), If
    ):
        raise ValueError(
            # Ensure we handle None as a valid value
            '%s.%s must be one of: "%s"'
            % (
                class_name,
                property,
                ", ".join(condition for condition in conditionals if condition),
            )
        )


elasticloadbalancingv2.one_of = my_one_of

# Fix troposphere/elasticloadbalancing.py LBCookieStickinessPolicy is a List and do not use class LBCookieStickinessPolicy
elasticloadbalancing.LoadBalancer.props["LBCookieStickinessPolicy"] = (
    [elasticloadbalancing.LBCookieStickinessPolicy],
    False,
)
elasticloadbalancing.LoadBalancer.props["Listeners"] = (
    [elasticloadbalancing.Listener],
    False,
)


# Fix to avoid setting OS variable TROPO_REAL_BOOL
def boolean(x):
    if x in [True, 1, "1", "true", "True"]:
        return True
    if x in [False, 0, "0", "false", "False"]:
        return False
    raise ValueError


validators.boolean = boolean


# RDSDBInstance reax check when using SourceDBInstanceIdentifier, forbidden properties can use If Condition to remove them
def validate_dbinstance(self):
    if "DBSnapshotIdentifier" not in self.properties:
        if "Engine" not in self.properties:
            raise ValueError(
                "Resource Engine is required in type %s" % self.resource_type
            )

    #    if "SourceDBInstanceIdentifier" in self.properties:
    #
    #        invalid_replica_properties = (
    #            "DBName",
    #            "MasterUsername",
    #            "MasterUserPassword",
    #            "PreferredBackupWindow",
    #            "MultiAZ",
    #            "DBSnapshotIdentifier",
    #        )
    #
    #        invalid_properties = [
    #            s for s in self.properties.keys() if s in invalid_replica_properties
    #        ]
    #
    #        if invalid_properties:
    #            raise ValueError(
    #                (
    #                    "{0} properties can't be provided when "
    #                    "SourceDBInstanceIdentifier is present "
    #                    "AWS::RDS::DBInstance."
    #                ).format(", ".join(sorted(invalid_properties)))
    #            )

    if (
        (
            "DBSnapshotIdentifier" not in self.properties
            and "SourceDBInstanceIdentifier" not in self.properties
        )
        and (
            "MasterUsername" not in self.properties
            or "MasterUserPassword" not in self.properties
        )
        and ("DBClusterIdentifier" not in self.properties)
    ):
        raise ValueError(
            r"Either (MasterUsername and MasterUserPassword) or"
            r" DBSnapshotIdentifier are required in type "
            r"AWS::RDS::DBInstance."
        )

    if "KmsKeyId" in self.properties and "StorageEncrypted" not in self.properties:
        raise ValueError(
            "If KmsKeyId is provided, StorageEncrypted is required "
            "AWS::RDS::DBInstance."
        )

    nonetype = type(None)
    avail_zone = self.properties.get("AvailabilityZone", None)
    multi_az = self.properties.get("MultiAZ", None)
    if not (
        isinstance(avail_zone, (AWSHelperFn, nonetype))
        and isinstance(multi_az, (AWSHelperFn, nonetype))
    ):
        if avail_zone and multi_az in [True, 1, "1", "true", "True"]:
            raise ValueError(
                "AvailabiltyZone cannot be set on "
                "DBInstance if MultiAZ is set to true."
            )

    storage_type = self.properties.get("StorageType", None)
    if storage_type and storage_type == "io1" and "Iops" not in self.properties:
        raise ValueError("Must specify Iops if using StorageType io1")

    allocated_storage = self.properties.get("AllocatedStorage")
    iops = self.properties.get("Iops", None)
    if iops and not isinstance(iops, AWSHelperFn):
        min_storage_size = 100
        engine = self.properties.get("Engine")
        if not isinstance(engine, AWSHelperFn) and engine.startswith("sqlserver"):
            min_storage_size = 20

        if (
            not isinstance(allocated_storage, AWSHelperFn)
            and int(allocated_storage) < min_storage_size
        ):
            raise ValueError(
                f"AllocatedStorage must be at least {min_storage_size} when "
                "Iops is set."
            )
        if (
            not isinstance(allocated_storage, AWSHelperFn)
            and not isinstance(iops, AWSHelperFn)
            and float(iops) / float(allocated_storage) > 50.0
        ):
            raise ValueError(
                "AllocatedStorage must be no less than " "1/50th the provisioned Iops"
            )


rds.DBInstance.validate = validate_dbinstance

# troposphere ec2 is quite old and SecurityGroupIngress is only a list
# without the obj type, this break auto_get_props, fix it overriding
ec2.SecurityGroup.props = {
    "GroupName": (str, False),
    "GroupDescription": (str, True),
    "SecurityGroupEgress": (list, False),
    "SecurityGroupIngress": ([ec2.SecurityGroupRule], False),
    "VpcId": (str, False),
    "Tags": ((Tags, list), False),
}
